import { PayloadInterface } from '../Interfaces/PayloadInterface';
import { PayloadSource } from '../Types/PayloadSource';
import { TransferPayload } from '../../TransferPayload/Interfaces/TransferPayload';
import { ItemContent } from '../../Content/ItemContent';
import { SyncResolvedParams, SyncResolvedPayload } from '../../../Runtime/Deltas/Utilities/SyncResolvedPayload';
import { PersistentSignatureData } from '../../../Runtime/Encryption/PersistentSignatureData';
export declare abstract class PurePayload<T extends TransferPayload<C>, C extends ItemContent = ItemContent> implements PayloadInterface<T> {
    readonly source: PayloadSource;
    readonly uuid: string;
    readonly content_type: string;
    readonly deleted: boolean;
    readonly content: C | string | undefined;
    readonly created_at: Date;
    readonly updated_at: Date;
    readonly created_at_timestamp: number;
    readonly updated_at_timestamp: number;
    readonly dirtyIndex?: number;
    readonly globalDirtyIndexAtLastSync?: number;
    readonly dirty?: boolean;
    readonly lastSyncBegan?: Date;
    readonly lastSyncEnd?: Date;
    readonly duplicate_of?: string;
    readonly user_uuid?: string;
    readonly key_system_identifier?: string | undefined;
    readonly shared_vault_uuid?: string | undefined;
    readonly last_edited_by_uuid?: string;
    readonly signatureData?: PersistentSignatureData;
    constructor(rawPayload: T, source?: PayloadSource);
    ejected(): TransferPayload;
    get serverUpdatedAt(): Date;
    get serverUpdatedAtTimestamp(): number;
    abstract copy(override?: Partial<TransferPayload>, source?: PayloadSource): this;
    abstract copyAsSyncResolved(override?: Partial<T> & SyncResolvedParams, source?: PayloadSource): SyncResolvedPayload;
}
